"use client";

import { DataServiceForm } from "@/components/DataServiceForm";
import { ErrorScreen } from "@/components/ErrorScreen";
import { ImportProcessTable } from "@/components/ImportProcessTable";
import { LoadingScreen } from "@/components/LoadingScreen";
import { CardList } from "@/components/cards/CardList";
import { ProjectCard } from "@/components/cards/ProjectCard";
import { Box, Container, Stack, Typography } from "@mui/material";
import { BreadcrumbsMenu } from "@/components/BreadcrumbsMenu";
import { ConfirmModal } from "@/components/ConfirmModal";
import { useState } from "react";
import { useRouter } from "next/navigation";
import { useGetDataService } from "@/hooks/useGetDataService";
import { Project } from "@/types";

export default function DataService({
  params: { eid },
}: {
  params: { eid: string };
}) {
  const parsedEid = parseInt(eid);

  const { data, loading, error } = useGetDataService(parsedEid);
  const router = useRouter();
  const [deleteModal, setDeleteModal] = useState<{
    visible: boolean;
    text?: string;
    eid?: number;
  }>({ visible: false });

  if (isNaN(parsedEid)) {
    return <ErrorScreen message={`Data Service '${eid}' inconnu`} />;
  }

  if (error) {
    return <ErrorScreen message={error} />;
  }

  if (loading) {
    return <LoadingScreen />;
  }

  if (!data) {
    return <ErrorScreen />;
  }

  // get projects that use this dataservice at least once
  // FIXME fix RQL to get projets only once juste
  const uniqueProjs: Project[] = [];
  const uniqueProjNames: string[] = [];
  data.projects?.map((p) => {
    if (!uniqueProjNames.includes(p.name)) {
      uniqueProjs.push(p);
      uniqueProjNames.push(p.name);
    }
  });

  return (
    <Container>
      <BreadcrumbsMenu page="Data Service" />
      <Stack spacing={2} alignItems={"center"}>
        <Box width="100%">
          <Typography variant="h3">Data Service: {data.name}</Typography>
        </Box>
        <Box width={"80%"} minWidth={300}>
          <DataServiceForm dataService={data} />
        </Box>
        <Stack width="100%">
          <Typography variant="h4">Projets</Typography>
          <Typography>
            Liste des projets utilisant ce data service dans l&apos;une de leur
            recettes
          </Typography>
        </Stack>
        <CardList loading={false} emptyText="Aucun projet">
          {uniqueProjs?.map((p, i) => (
            <ProjectCard
              key={i}
              name={p.name}
              activated={p.activated}
              virtuoso_url={p.virtuoso_url}
              canEdit={false}
              onOpen={() => {
                router.push("/project/" + p.eid);
              }}
              onDelete={() =>
                setDeleteModal({
                  visible: true,
                  text: `Voulez vous vraiment supprimer le projet '${p.name}' ?`,
                  eid: p.eid,
                })
              }
            />
          ))}
        </CardList>
        <Box width="100%">
          <Typography variant="h4">Import process</Typography>
        </Box>
        <ImportProcessTable dataServiceEid={parsedEid} />
      </Stack>
      <ConfirmModal
        open={deleteModal.visible}
        text={deleteModal?.text ?? ""}
        onClose={() => setDeleteModal((prev) => ({ ...prev, visible: false }))}
        onAccept={() => {
          return new Promise((resolve) => {
            setTimeout(resolve, 1000);
          });
        }}
      />
    </Container>
  );
}
